@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
- * Robert Lougher <rob@lougher.org.uk>.
+ * Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -194,9 +194,8 @@ Class *defineClass(char *classname, char *data, int offset, int len,
     READ_U2(cp_count, ptr, len);
 
     constant_pool = &classblock->constant_pool;
-    constant_pool->type = (u1 *)sysMalloc(cp_count);
-    constant_pool->info = (ConstantPoolEntry *)
-                       sysMalloc(cp_count*sizeof(ConstantPoolEntry));
+    constant_pool->type = sysMalloc(cp_count);
+    constant_pool->info = sysMalloc(cp_count*sizeof(ConstantPoolEntry));
 
     for(i = 1; i < cp_count; i++) {
         u1 tag;
@@ -223,19 +222,11 @@ Class *defineClass(char *classname, char *data, int offset, int len,
                break;
            }
 
+           case CONSTANT_Float:
            case CONSTANT_Integer:
                READ_U4(CP_INFO(constant_pool,i), ptr, len);
                break;
 
-           case CONSTANT_Float:
-           {
-               u4 val;
-
-               READ_U4(val, ptr, len);
-               CP_INFO(constant_pool,i) = FLOAT_CONST(val);
-               break;
-           }
-
            case CONSTANT_Long:
                READ_U8(*(u8 *)&(CP_INFO(constant_pool,i)), ptr, len);
                CP_TYPE(constant_pool,++i) = 0;
@@ -305,8 +296,7 @@ Class *defineClass(char *classname, char *data, int offset, int len,
     classblock->class_loader = class_loader;
 
     READ_U2(intf_count = classblock->interfaces_count, ptr, len);
-    interfaces = classblock->interfaces =
-                      (Class **)sysMalloc(intf_count * sizeof(Class *));
+    interfaces = classblock->interfaces = sysMalloc(intf_count * sizeof(Class *));
 
     memset(interfaces, 0, intf_count * sizeof(Class *));
     for(i = 0; i < intf_count; i++) {
@@ -318,8 +308,7 @@ Class *defineClass(char *classname, char *data, int offset, int len,
     }
 
     READ_U2(classblock->fields_count, ptr, len);
-    classblock->fields = (FieldBlock *)
-                     sysMalloc(classblock->fields_count * sizeof(FieldBlock));
+    classblock->fields = sysMalloc(classblock->fields_count * sizeof(FieldBlock));
 
     for(i = 0; i < classblock->fields_count; i++) {
         u2 name_idx, type_idx;
@@ -364,8 +353,7 @@ Class *defineClass(char *classname, char *data, int offset, int len,
 
     READ_U2(classblock->methods_count, ptr, len);
 
-    classblock->methods = (MethodBlock *)
-            sysMalloc(classblock->methods_count * sizeof(MethodBlock));
+    classblock->methods = sysMalloc(classblock->methods_count * sizeof(MethodBlock));
 
     memset(classblock->methods, 0, classblock->methods_count * sizeof(MethodBlock));
 
@@ -402,15 +390,14 @@ Class *defineClass(char *classname, char *data, int offset, int len,
                 READ_U2(method->max_locals, ptr, len);
 
                 READ_U4(code_length, ptr, len);
-                method->code = (char *)sysMalloc(code_length);
+                method->code = sysMalloc(code_length);
                 memcpy(method->code, ptr, code_length);
                 ptr += code_length;
 
                 method->code_size = code_length;
 
                 READ_U2(method->exception_table_size, ptr, len);
-                method->exception_table = (ExceptionTableEntry *)
-                sysMalloc(method->exception_table_size*sizeof(ExceptionTableEntry));
+                method->exception_table = sysMalloc(method->exception_table_size*sizeof(ExceptionTableEntry));
 
                 for(j = 0; j < method->exception_table_size; j++) {
                     ExceptionTableEntry *entry = &method->exception_table[j];              
@@ -432,8 +419,7 @@ Class *defineClass(char *classname, char *data, int offset, int len,
 
                     if(attr_name == SYMBOL(LineNumberTable)) {
                         READ_U2(method->line_no_table_size, ptr, len);
-                        method->line_no_table = (LineNoTableEntry *)
-                            sysMalloc(method->line_no_table_size*sizeof(LineNoTableEntry));
+                        method->line_no_table = sysMalloc(method->line_no_table_size*sizeof(LineNoTableEntry));
 
                         for(j = 0; j < method->line_no_table_size; j++) {
                             LineNoTableEntry *entry = &method->line_no_table[j];              
@@ -449,7 +435,7 @@ Class *defineClass(char *classname, char *data, int offset, int len,
                     int j;
 
                     READ_U2(method->throw_table_size, ptr, len);
-                    method->throw_table = (u2 *)sysMalloc(method->throw_table_size*sizeof(u2));
+                    method->throw_table = sysMalloc(method->throw_table_size*sizeof(u2));
                     for(j = 0; j < method->throw_table_size; j++) {
                         READ_U2(method->throw_table[j], ptr, len);
                     }
@@ -881,7 +867,7 @@ void linkClass(Class *class) {
    if(cb->state >= CLASS_LINKED)
        return;
 
-   objectLock((Object *)class);
+   objectLock(class);
 
    if(cb->state >= CLASS_LINKED)
        goto unlock;
@@ -933,7 +919,7 @@ void linkClass(Class *class) {
            /* set up native invoker to wrapper to resolve function 
               on first invocation */
 
-           mb->native_invoker = (void *) resolveNativeWrapper;
+           mb->native_invoker = &resolveNativeWrapper;
 
            /* native methods have no code attribute so these aren't filled
               in at load time - as these values are used when creating frame
@@ -976,7 +962,7 @@ void linkClass(Class *class) {
    method_table_size = spr_mthd_tbl_sze + new_methods_count;
 
    if(!(cb->access_flags & ACC_INTERFACE)) {
-       method_table = (MethodBlock**)sysMalloc(method_table_size * sizeof(MethodBlock*));
+       method_table = sysMalloc(method_table_size * sizeof(MethodBlock*));
 
        /* Copy parents method table to the start */
        memcpy(method_table, spr_mthd_tbl, spr_mthd_tbl_sze * sizeof(MethodBlock*));
@@ -998,7 +984,7 @@ void linkClass(Class *class) {
        new_itable_count += CLASS_CB(cb->interfaces[i])->imethod_table_size;
 
    cb->imethod_table_size = spr_imthd_tbl_sze + new_itable_count;
-   cb->imethod_table = (ITableEntry*)sysMalloc(cb->imethod_table_size * sizeof(ITableEntry));
+   cb->imethod_table = sysMalloc(cb->imethod_table_size * sizeof(ITableEntry));
 
    /* copy parent's interface table - the offsets into the method table won't change */
 
@@ -1027,7 +1013,7 @@ void linkClass(Class *class) {
    /* if we're an interface all finished - offsets aren't used */
 
    if(!(cb->access_flags & ACC_INTERFACE)) {
-       int *offsets_pntr = (int*)sysMalloc(itbl_offset_count * sizeof(int));
+       int *offsets_pntr = sysMalloc(itbl_offset_count * sizeof(int));
        int old_mtbl_size = method_table_size;
        MethodBlock *miranda[MRNDA_CACHE_SZE];
        int miranda_count = 0;
@@ -1196,7 +1182,7 @@ void linkClass(Class *class) {
    cb->state = CLASS_LINKED;
 
 unlock:
-   objectUnlock((Object *)class);
+   objectUnlock(class);
 }
 
 Class *initClass(Class *class) {
@@ -1211,27 +1197,27 @@ Class *initClass(Class *class) {
       return class;
 
    linkClass(class);
-   objectLock((Object *)class);
+   objectLock(class);
 
    while(cb->state == CLASS_INITING)
       if(cb->initing_tid == threadSelf()->id) {
-         objectUnlock((Object *)class);
+         objectUnlock(class);
          return class;
       } else {
           /* FALSE means this wait is non-interruptible.
              An interrupt will appear as if the initialiser
              failed (below), and clearing will lose the
              interrupt status */
-          objectWait0((Object *)class, 0, 0, FALSE);
+          objectWait0(class, 0, 0, FALSE);
       }
 
    if(cb->state >= CLASS_INITED) {
-      objectUnlock((Object *)class);
+      objectUnlock(class);
       return class;
    }
 
    if(cb->state == CLASS_BAD) {
-       objectUnlock((Object *)class);
+       objectUnlock(class);
        signalException(java_lang_NoClassDefFoundError, cb->name);
        return NULL;
    }
@@ -1239,7 +1225,7 @@ Class *initClass(Class *class) {
    cb->state = CLASS_INITING;
    cb->initing_tid = threadSelf()->id;
 
-   objectUnlock((Object *)class);
+   objectUnlock(class);
 
    if(!(cb->access_flags & ACC_INTERFACE) && cb->super
               && (CLASS_CB(cb->super)->state != CLASS_INITED)) {
@@ -1291,11 +1277,11 @@ Class *initClass(Class *class) {
        state = CLASS_INITED;
    
 set_state_and_notify:
-   objectLock((Object *)class);
+   objectLock(class);
    cb->state = state;
 
-   objectNotifyAll((Object *)class);
-   objectUnlock((Object *)class);
+   objectNotifyAll(class);
+   objectUnlock(class);
 
    return state == CLASS_BAD ? NULL : class;
 }
@@ -1312,7 +1298,7 @@ char *findFileEntry(char *path, int *file_len) {
     *file_len = ftell(fd);
     fseek(fd, 0L, SEEK_SET);
 
-    data = (char *)sysMalloc(*file_len);
+    data = sysMalloc(*file_len);
     read_len = fread(data, sizeof(char), *file_len, fd);
     fclose(fd);
 
@@ -1666,9 +1652,9 @@ void threadBootClasses() {
 #undef ITERATE
 #define ITERATE(ptr)                                         \
     if(CLASS_CB((Class *)ptr)->class_loader == class_loader) \
-        markObject(ptr, mark, mark_soft_refs)
+        markObject(ptr, mark)
 
-void markLoaderClasses(Object *class_loader, int mark, int mark_soft_refs) {
+void markLoaderClasses(Object *class_loader, int mark) {
     Object *vmdata = INST_DATA(class_loader, Object*, ldr_vmdata_offset);
 
     if(vmdata != NULL) {
@@ -1678,7 +1664,7 @@ void markLoaderClasses(Object *class_loader, int mark, int mark_soft_refs) {
 }
 
 #undef ITERATE
-#define ITERATE(ptr)  threadReference((Object**)ptr)
+#define ITERATE(ptr) threadReference((Object**)ptr)
 
 void threadLoaderClasses(Object *class_loader) {
     Object *vmdata = INST_DATA(class_loader, Object*, ldr_vmdata_offset);
@@ -1806,7 +1792,7 @@ int parseBootClassPath(char *cp_var) {
     int i, j, len, max = 0;
     struct stat info;
 
-    cp = (char*)sysMalloc(strlen(cp_var)+1);
+    cp = sysMalloc(strlen(cp_var)+1);
     strcpy(cp, cp_var);
 
     for(i = 0, start = pntr = cp; *pntr; pntr++) {
@@ -1821,7 +1807,7 @@ int parseBootClassPath(char *cp_var) {
     if(start != pntr)
         i++;
 
-    bootclasspath = (BCPEntry *)sysMalloc(sizeof(BCPEntry)*i);
+    bootclasspath = sysMalloc(sizeof(BCPEntry)*i);
 
     for(j = 0, pntr = cp; i > 0; i--) {
         while(*pntr == ':')
@@ -1897,8 +1883,8 @@ void scanDirForJars(char *dir) {
 void scanDirsForJars(char *directories) {
     int dirslen = strlen(directories);
     char *pntr, *end, *dirs = sysMalloc(dirslen + 1);
-    strcpy(dirs, directories);
 
+    strcpy(dirs, directories);
     for(end = pntr = &dirs[dirslen]; pntr != dirs; pntr--) {
         if(*pntr == ':') {
             char *start = pntr + 1;
