@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
- * Robert Lougher <rob@lougher.org.uk>.
+ * Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -58,7 +58,7 @@ void *lookupLoadedDlls(MethodBlock *mb);
 
 char *mangleString(char *utf8) {
     int len = utf8Len(utf8);
-    unsigned short *unicode = (unsigned short*) sysMalloc(len * 2);
+    unsigned short *unicode = sysMalloc(len * 2);
     char *mangled, *mngldPtr;
     int i, mangledLen = 0;
 
@@ -81,7 +81,7 @@ char *mangleString(char *utf8) {
         }
     }
 
-    mangled = mngldPtr = (char*) sysMalloc(mangledLen + 1);
+    mangled = mngldPtr = sysMalloc(mangledLen + 1);
 
     /* Construct the mangled string */
 
@@ -142,7 +142,7 @@ char *mangleSignature(MethodBlock *mb) {
     /* find ending ) */
     for(i = strlen(type) - 1; type[i] != ')'; i--);
 
-    nonMangled = (char *) sysMalloc(i);
+    nonMangled = sysMalloc(i);
     strncpy(nonMangled, type + 1, i - 1);
     nonMangled[i - 1] = '\0';
     
@@ -173,7 +173,7 @@ void *lookupInternal(MethodBlock *mb) {
                 jam_printf("internal");
 
             /* Found it -- set the invoker to the native method */
-            return mb->native_invoker = (void*)methods[i].method;
+            return mb->native_invoker = methods[i].method;
         }
     }
 
@@ -261,10 +261,15 @@ int resolveDll(char *name, Object *loader) {
         DllEntry *dll2;
         void *onload, *handle = nativeLibOpen(name);
 
-        if(handle == NULL)
-            return FALSE;
+        if(handle == NULL) {
+            if(verbose) {
+                char *error = nativeLibError();
 
-        TRACE("<DLL: Successfully opened library %s>\n", name);
+                jam_printf("[Failed to open library %s: %s]\n", name,
+                           error == NULL ? "<no reason available>" : error);
+            }
+            return FALSE;
+        }
 
         if((onload = nativeLibSym(handle, "JNI_OnLoad")) != NULL) {
             int ver;
@@ -273,15 +278,19 @@ int resolveDll(char *name, Object *loader) {
             ver = (*(jint (*)(JavaVM*, void*))onload)(&invokeIntf, NULL);
 
             if(ver != JNI_VERSION_1_2 && ver != JNI_VERSION_1_4) {
-                TRACE("<DLL: JNI_OnLoad returned unsupported version %d.\n>",
-                      ver);
+                if(verbose)
+                    jam_printf("[%s: JNI_OnLoad returned unsupported version"
+                               " number %d.\n>", name, ver);
 
                 return FALSE;
             }
         }
 
-        dll = (DllEntry*)sysMalloc(sizeof(DllEntry));
-        dll->name = strcpy((char*)sysMalloc(strlen(name)+1), name);
+        if(verbose)
+           jam_printf("[Opened native library %s]\n", name);
+
+        dll = sysMalloc(sizeof(DllEntry));
+        dll->name = strcpy(sysMalloc(strlen(name) + 1), name);
         dll->handle = handle;
         dll->loader = loader;
 
@@ -306,8 +315,11 @@ int resolveDll(char *name, Object *loader) {
             newLibraryUnloader(loader, dll);
 
     } else
-        if(dll->loader != loader)
+        if(dll->loader != loader) {
+            if(verbose)
+                jam_printf("[%s: already loaded by another classloader]\n");
             return FALSE;
+        }
 
     return TRUE;
 }
@@ -449,7 +461,7 @@ void *lookupLoadedDlls(MethodBlock *mb) {
 
         mb->code = (unsigned char *) func;
         mb->native_extra_arg = nativeExtraArg(mb);
-        return mb->native_invoker = (void*) callJNIWrapper;
+        return mb->native_invoker = &callJNIWrapper;
     }
 
     return NULL;
