@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
- * Robert Lougher <rob@lougher.org.uk>.
+ * Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -335,7 +335,7 @@ Object *convertStackTrace(Object *vmthrwble) {
    In rare circumstances a stack backtrace may hold the only
    reference to a class */
 
-void markVMThrowable(Object *vmthrwble, int mark, int mark_soft_refs) {
+void markVMThrowable(Object *vmthrwble, int mark) {
     Object *array;
 
     if((array = INST_DATA(vmthrwble, Object*, backtrace_offset)) != NULL) {
@@ -344,7 +344,7 @@ void markVMThrowable(Object *vmthrwble, int mark, int mark_soft_refs) {
 
         for(i = 0; i < depth; i += 2) {
             MethodBlock *mb = (MethodBlock*)src[i];
-            markObject(mb->class, mark, mark_soft_refs);
+            markObject(mb->class, mark);
         }
     }
 }
