@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
- * Robert Lougher <rob@lougher.org.uk>.
+ * Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -548,12 +548,12 @@ retry:
                         i = high - low + 4;
 #endif
                     } else {
-                        SwitchTable *table = (SwitchTable*) sysMalloc(sizeof(SwitchTable));
+                        SwitchTable *table = sysMalloc(sizeof(SwitchTable));
 
                         table->low = low;
                         table->high = high; 
                         table->deflt = &new_code[map[pc + deflt]];
-                        table->entries = (Instruction**) sysMalloc((high - low + 1) * sizeof(Instruction *));
+                        table->entries = sysMalloc((high - low + 1) * sizeof(Instruction *));
 
                         for(i = 3; i < (high - low + 4); i++)
                             table->entries[i - 3] = &new_code[map[pc + ntohl(aligned_pc[i])]];
@@ -606,11 +606,11 @@ retry:
                         i = npairs*2+2;
 #endif
                     } else {
-                        LookupTable *table = (LookupTable*)sysMalloc(sizeof(LookupTable));
+                        LookupTable *table = sysMalloc(sizeof(LookupTable));
    
                         table->num_entries = npairs;
                         table->deflt = &new_code[map[pc + deflt]];
-                        table->entries = (LookupEntry*)sysMalloc(npairs * sizeof(LookupEntry));
+                        table->entries = sysMalloc(npairs * sizeof(LookupEntry));
                             
                         for(i = 2, j = 0; j < npairs; i += 2, j++) {
                             table->entries[j].key = ntohl(aligned_pc[i]);
@@ -941,7 +941,7 @@ retry:
     enableSuspend(self);
 
     /* We don't need the old bytecode stream anymore */
-    if(!mb->access_flags & ACC_ABSTRACT)
+    if(!(mb->access_flags & ACC_ABSTRACT))
         sysFree(code);
 }
 #endif
