@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007, 2008, 2009 Robert Lougher <rob@lougher.org.uk>.
+ * Copyright (C) 2007, 2008, 2009 Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -927,7 +927,7 @@ void inlineBlock(MethodBlock *mb, BasicBlock *block, Thread *self) {
        inlineBlocks doesn't free the blocks as previously scanned
        blocks may be inlined later */
 
-    while(start != end->next) {
+    for(end = end->next; start != end; ) {
         BasicBlock *next = start->next;
 
         sysFree(start->opcodes);
