@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
- * Robert Lougher <rob@lougher.org.uk>.
+ * Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -495,7 +495,11 @@ typedef struct method_annotation_data {
     AnnotationData *dft_val;
 } MethodAnnotationData;
 
-typedef struct methodblock {
+typedef struct methodblock MethodBlock;
+
+typedef uintptr_t *(*NativeMethod)(Class*, struct methodblock*, uintptr_t*);
+
+struct methodblock {
    Class *class;
    char *name;
    char *type;
@@ -508,7 +512,7 @@ typedef struct methodblock {
    u2 exception_table_size;
    u2 line_no_table_size;
    int native_extra_arg;
-   void *native_invoker;
+   NativeMethod native_invoker;
    void *code;
    int code_size;
    u2 *throw_table;
@@ -520,7 +524,7 @@ typedef struct methodblock {
    QuickPrepareInfo *quick_prepare_info;
    ProfileInfo *profile_info;
 #endif
-} MethodBlock;
+};
 
 typedef struct fieldblock {
    Class *class;
@@ -759,9 +763,6 @@ extern Object *allocTypeArray(int type, int size);
 extern Object *allocArray(Class *class, int size, int el_size);
 extern Object *allocMultiArray(Class *array_class, int dim, intptr_t *count);
 extern Object *cloneObject(Object *ob);
-extern void markRoot(Object *ob);
-extern void markConservativeRoot(Object *ob);
-extern void markObject(Object *ob, int mark, int mark_soft_refs);
 extern uintptr_t getObjectHashcode(Object *ob);
 
 extern void gc1();
@@ -791,8 +792,14 @@ extern void registerStaticObjectRefLocked(Object **ref, Object *obj);
 extern void gcPendingFree(void *addr);
 
 /* GC support */
-extern void threadReference(Object **ref);
+extern void markRoot(Object *ob);
+extern void markConservativeRoot(Object *ob);
+extern void markObject(Object *ob, int mark);
+extern void markJNIClearedWeakRef(Object *ob);
+extern void markJNIGlobalRef(Object *ob);
+extern int isMarkedJNIWeakGlobalRef(Object *ob);
 extern int isMarked(Object *ob);
+extern void threadReference(Object **ref);
 
 /* Class */
 
@@ -830,7 +837,7 @@ extern char *getClassPath();
 extern char *getBootClassPath();
 
 extern void markBootClasses();
-extern void markLoaderClasses(Object *loader, int mark, int mark_soft_refs);
+extern void markLoaderClasses(Object *loader, int mark);
 extern void threadBootClasses();
 extern void threadLoaderClasses(Object *class_loader);
 extern void newLibraryUnloader(Object *class_loader, void *entry);
@@ -889,7 +896,7 @@ extern CodePntr findCatchBlock(Class *exception);
 extern Object *setStackTrace0(ExecEnv *ee, int max_depth);
 extern Object *convertStackTrace(Object *vmthrwble);
 extern int mapPC2LineNo(MethodBlock *mb, CodePntr pc_pntr);
-extern void markVMThrowable(Object *vmthrwble, int mark, int mark_soft_refs);
+extern void markVMThrowable(Object *vmthrwble, int mark);
 extern void initialiseException();
 
 #define exceptionOccurred0(ee) \
@@ -969,6 +976,7 @@ extern void threadLiveClassLoaderDlls();
 /* OS */
 
 extern char *nativeLibPath();
+extern char *nativeLibError();
 extern void *nativeLibOpen(char *path);
 extern void nativeLibClose(void *handle);
 extern char *nativeLibMapName(char *name);
@@ -999,6 +1007,8 @@ extern int initJNILrefs();
 extern void initialiseJNI();
 extern void *getJNIInterface();
 extern void markJNIGlobalRefs();
+extern void scanJNIWeakGlobalRefs();
+extern void markJNIClearedWeakRefs();
 
 /* properties */
 
@@ -1060,3 +1070,7 @@ extern void getTimeoutAbsolute(struct timespec *ts, long long millis,
 extern void getTimeoutRelative(struct timespec *ts, long long millis,
                         long long nanos);
 
+/* sig */
+
+extern int sigElement2Size(char element);
+
