@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008
- * Robert Lougher <rob@lougher.org.uk>.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
+ * Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -209,9 +209,10 @@ int monitorWait0(Monitor *mon, Thread *self, long long ms, int ns,
     if(interruptible && self->interrupted)
         interrupted = TRUE;
     else {
-        if(blocked)
+        if(blocked) {
+            self->blocked_mon = mon;
             self->blocked_count++;
-        else
+        } else
             self->waited_count++;
 
         self->interrupting = FALSE;
@@ -260,6 +261,9 @@ int monitorWait0(Monitor *mon, Thread *self, long long ms, int ns,
     self->state = RUNNING;
     self->wait_mon = NULL;
 
+    if(blocked)
+        self->blocked_mon = NULL;
+
    /* Restore the monitor owner and recursion count */
     mon->count = old_count;
     mon->owner = self;
@@ -303,7 +307,7 @@ Monitor *allocMonitor(Object *obj) {
         mon = mon_free_list;
         mon_free_list = mon->next;      
     } else {
-        mon = (Monitor *)sysMalloc(sizeof(Monitor));
+        mon = sysMalloc(sizeof(Monitor));
         monitorInit(mon);
     }
     mon->obj = obj;
@@ -342,7 +346,7 @@ void objectLock(Object *obj) {
     TRACE("Thread %p lock on obj %p...\n", self, obj);
 
     if(LOCKWORD_COMPARE_AND_SWAP(&obj->lock, 0, thin_locked)) {
-        /* This barrier is not needed for the thin-locking implementation --
+        /* This barrier is not needed for the thin-locking implementation;
            it's a requirement of the Java memory model. */
         JMM_LOCK_MBARRIER();
         return;
@@ -375,14 +379,16 @@ try_again2:
 
     if(mon->obj != obj) {
         while(entering = LOCKWORD_READ(&mon->entering),
-                        !(LOCKWORD_COMPARE_AND_SWAP(&mon->entering, entering, entering-1)));
+                        !(LOCKWORD_COMPARE_AND_SWAP(&mon->entering,
+                                                    entering, entering-1)));
         goto try_again;
     }
 
     monitorLock(mon, self);
 
     while(entering = LOCKWORD_READ(&mon->entering),
-                    !(LOCKWORD_COMPARE_AND_SWAP(&mon->entering, entering, entering-1)));
+                    !(LOCKWORD_COMPARE_AND_SWAP(&mon->entering,
+                                                entering, entering-1)));
 
     while((LOCKWORD_READ(&obj->lock) & SHAPE_BIT) == 0) {
         setFlcBit(obj);
@@ -402,13 +408,13 @@ void objectUnlock(Object *obj) {
     TRACE("Thread %p unlock on obj %p...\n", self, obj);
 
     if(lockword == thin_locked) {
-        /* This barrier is not needed for the thin-locking implementation --
+        /* This barrier is not needed for the thin-locking implementation;
            it's a requirement of the Java memory model. */
         JMM_UNLOCK_MBARRIER();
         LOCKWORD_WRITE(&obj->lock, 0);
 
         /* Required by thin-locking mechanism. */
-        UNLOCK_MBARRIER();
+        MBARRIER();
 
 retry:
         if(testFlcBit(obj)) {
@@ -435,8 +441,9 @@ retry:
                                 (mon->in_wait == 0)) {
                     TRACE("Thread %p is deflating obj %p...\n", self, obj);
 
-                    /* This barrier is not needed for the thin-locking implementation --
-                       it's a requirement of the Java memory model. */
+                    /* This barrier is not needed for the thin-locking
+                       implementation; it's a requirement of the Java
+                       memory model. */
                     JMM_UNLOCK_MBARRIER();
 
                     LOCKWORD_WRITE(&obj->lock, 0);
@@ -471,7 +478,8 @@ void objectWait0(Object *obj, long long ms, int ns, int interruptible) {
         return;
 
 not_owner:
-    signalException(java_lang_IllegalMonitorStateException, "thread not owner");
+    signalException(java_lang_IllegalMonitorStateException,
+                    "thread not owner");
 }
 
 void objectNotify(Object *obj) {
@@ -490,7 +498,8 @@ void objectNotify(Object *obj) {
             return;
     }
 
-    signalException(java_lang_IllegalMonitorStateException, "thread not owner");
+    signalException(java_lang_IllegalMonitorStateException,
+                    "thread not owner");
 }
 
 void objectNotifyAll(Object *obj) {
@@ -509,7 +518,8 @@ void objectNotifyAll(Object *obj) {
             return;
     }
 
-    signalException(java_lang_IllegalMonitorStateException, "thread not owner");
+    signalException(java_lang_IllegalMonitorStateException,
+                    "thread not owner");
 }
 
 int objectLockedByCurrent(Object *obj) {
@@ -528,6 +538,20 @@ int objectLockedByCurrent(Object *obj) {
     return FALSE;
 }
 
+Thread *objectLockedBy(Object *obj) {
+    uintptr_t lockword = LOCKWORD_READ(&obj->lock);
+    Thread *owner;
+
+    if((lockword & SHAPE_BIT) == 0) {
+        int tid = (lockword&TID_MASK)>>TID_SHIFT;
+        owner = findRunningThreadByTid(tid);
+    } else {
+        Monitor *mon = (Monitor*) (lockword & ~SHAPE_BIT);
+        owner = mon->owner;
+    }
+    return owner;
+}
+
 void initialiseMonitor() {
     /* Init hash table, create lock */
     initHashTable(mon_cache, HASHTABSZE, TRUE);
