@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
- * Robert Lougher <rob@lougher.org.uk>.
+ * Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -36,7 +36,6 @@
 #include "natives.h"
 #include "symbol.h"
 #include "excep.h"
-
 #include "reflect.h"
 
 static int pd_offset;
@@ -119,32 +118,7 @@ uintptr_t *arraycopy(Class *class, MethodBlock *mb, uintptr_t *ostack) {
         }
 
         if(isInstanceOf(dest->class, src->class)) {
-            int size;
-
-            switch(scb->name[1]) {
-                case 'B':
-                case 'Z':
-                    size = 1;
-                    break;
-                case 'C':
-                case 'S':
-                    size = 2;
-                    break;
-                case 'I':
-                case 'F':
-                    size = 4;
-                    break;
-                case 'L':
-                case '[':
-                    size = sizeof(Object*);
-                    break;
-                case 'J':
-                case 'D':
-		default:
-                    size = 8;
-                    break;
-            } 
-
+            int size = sigElement2Size(scb->name[1]);
             memmove(ddata + start2*size, sdata + start1*size, length*size);
         } else {
             Object **sob, **dob;
@@ -236,7 +210,7 @@ uintptr_t *nativeLoad(Class *class, MethodBlock *mb, uintptr_t *ostack) {
     ostack[0] = resolveDll(name, class_loader);
     sysFree(name);
 
-    return ostack+1;
+    return ostack + 1;
 }
 
 uintptr_t *mapLibraryName(Class *class, MethodBlock *mb, uintptr_t *ostack) {
@@ -354,8 +328,8 @@ uintptr_t *getClassSignature(Class *class, MethodBlock *mb, uintptr_t *ostack) {
 
 uintptr_t *getSuperclass(Class *class, MethodBlock *mb, uintptr_t *ostack) {
     ClassBlock *cb = CLASS_CB((Class*)ostack[0]);
-    *ostack++ = (uintptr_t) (IS_PRIMITIVE(cb) || IS_INTERFACE(cb) ? NULL
-                                                                  : cb->super);
+    *ostack++ = (uintptr_t) (IS_PRIMITIVE(cb) ||
+                             IS_INTERFACE(cb) ? NULL : cb->super);
     return ostack;
 }
 
@@ -672,6 +646,7 @@ uintptr_t *defineClass0(Class *clazz, MethodBlock *mb, uintptr_t *ostack) {
                 if(cstr[i]=='.') cstr[i]='/';
 
             class = defineClass(cstr, data, offset, data_len, class_loader);
+
             if(class != NULL) {
                 INST_DATA(class, uintptr_t, pd_offset) = pd;
                 linkClass(class);
@@ -696,7 +671,7 @@ uintptr_t *findLoadedClass(Class *clazz, MethodBlock *mb, uintptr_t *ostack) {
         return ostack;
     }
 
-    cstr = String2Cstr(string);
+    cstr = String2Utf8(string);
     len = strlen(cstr);
 
     for(i = 0; i < len; i++)
@@ -1334,15 +1309,27 @@ uintptr_t *getThreadInfoForId(Class *class, MethodBlock *mb,
     Object *info = NULL;
 
     if(thread != NULL) {
+        Class *helper_class = findSystemClass("jamvm/ThreadInfoHelper");
         Class *info_class = findSystemClass("java/lang/management/ThreadInfo");
 
-        if(info_class != NULL) {
+        if(info_class != NULL && helper_class != NULL) {
+            MethodBlock *helper = findMethod(helper_class,
+                                             newUtf8("createThreadInfo"),
+                                             newUtf8("(Ljava/lang/Thread;"
+                                                     "Ljava/lang/Object;"
+                                                     "Ljava/lang/Thread;)"
+                                                     "[Ljava/lang/Object;"));
             MethodBlock *init = findMethod(info_class, SYMBOL(object_init),
-                                   newUtf8("(Ljava/lang/Thread;"
-                                           "JJLjava/lang/Object;"
-                                           "Ljava/lang/Thread;JJZZ"
-                                           "[Ljava/lang/StackTraceElement;)V"));
-            if(init != NULL) {
+                                 newUtf8("(JLjava/lang/String;"
+                                         "Ljava/lang/Thread$State;"
+                                         "JJLjava/lang/String;"
+                                         "JLjava/lang/String;JJZZ"
+                                         "[Ljava/lang/StackTraceElement;"
+                                         "[Ljava/lang/management/MonitorInfo;"
+                                         "[Ljava/lang/management/LockInfo;)V"));
+
+
+            if(init != NULL && helper != NULL) {
                 Frame *last;
                 int in_native;
                 Object *vmthrowable;
@@ -1361,20 +1348,35 @@ uintptr_t *getThreadInfoForId(Class *class, MethodBlock *mb,
                     resumeThread(thread);
 
                 if(vmthrowable != NULL) {
-                    Object *trace;
+                    Object **helper_info, *trace;
+
                     if((info = allocObject(info_class)) != NULL &&
                              (trace = convertStackTrace(vmthrowable)) != NULL) {
 
                         Monitor *mon = thread->blocked_mon;
                         Object *lock = mon != NULL ? mon->obj : NULL;
-                        Thread *owner = mon != NULL ? mon->owner : NULL;
+                        Thread *owner = lock != NULL ?
+                                             objectLockedBy(lock) : NULL;
                         Object *lock_owner = owner != NULL ?
                                                owner->ee->thread : NULL;
+                        long long owner_id = owner != NULL ?
+                                               javaThreadId(owner) : -1;
 
-                        executeMethod(info, init, thread->ee->thread,
-                                      thread->blocked_count, 0LL, lock,
-                                      lock_owner, thread->waited_count,
-                                      0LL, in_native, FALSE, trace);
+                        helper_info = executeStaticMethod(helper_class,
+                                              helper, thread->ee->thread, lock,
+                                              lock_owner);
+
+                        if(!exceptionOccurred()) {
+                            helper_info = ARRAY_DATA(*helper_info, Object*);
+
+                            executeMethod(info, init, id,
+                                          helper_info[0], helper_info[1],
+                                          thread->blocked_count, 0LL,
+                                          helper_info[2], owner_id,
+                                          helper_info[3], thread->waited_count,
+                                          0LL, in_native, FALSE, trace,
+                                          NULL, NULL);
+                        }
                     }
                 }
             }
