@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008
- * Robert Lougher <rob@lougher.org.uk>.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
+ * Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -53,6 +53,10 @@ int nativeAvailableProcessors() {
 #endif
 }
 
+char *nativeLibError() {
+    return dlerror();
+}
+
 char *nativeLibPath() {
     return getenv("LD_LIBRARY_PATH");
 }
