@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008
- * Robert Lougher <rob@lougher.org.uk>.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
+ * Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -27,17 +27,7 @@
 
 #include "jam.h"
 #include "symbol.h"
-
-/* If we have endian.h include it.  Otherwise, include sys/param.h
-   if we have it. If the BYTE_ORDER macro is still undefined, we
-   fall-back, and work out the endianness ourselves at runtime --
-   this always works.
-*/
-#ifdef HAVE_ENDIAN_H
-#include <endian.h>
-#elif HAVE_SYS_PARAM_H
-#include <sys/param.h>
-#endif
+#include "properties.h"
 
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -66,7 +56,7 @@ void setProperty(Object *properties, char *key, char *value) {
     Object *v = Cstr2String(value ? value : "?");
 
     MethodBlock *mb = lookupMethod(properties->class, SYMBOL(put),
-                           SYMBOL(_java_lang_Object_java_lang_Object__java_lang_Object));
+                  SYMBOL(_java_lang_Object_java_lang_Object__java_lang_Object));
     executeMethod(properties, mb, k, v);
 }
 
@@ -75,7 +65,8 @@ void addCommandLineProperties(Object *properties) {
         int i;
 
         for(i = 0; i < commandline_props_count; i++) {
-            setProperty(properties, commandline_props[i].key, commandline_props[i].value);
+            setProperty(properties, commandline_props[i].key,
+                                    commandline_props[i].value);
             sysFree(commandline_props[i].key);
         }
 
@@ -113,26 +104,6 @@ void setLocaleProperties(Object *properties) {
 #endif
 }
 
-void setEndianProperty(Object *properties) {
-#ifdef BYTE_ORDER
-#if BYTE_ORDER == BIG_ENDIAN
-    setProperty(properties, "gnu.cpu.endian", "big");
-#else
-    setProperty(properties, "gnu.cpu.endian", "little");
-#endif
-#else
-    /* No byte-order macro -- work it out ourselves at runtime */
-    union {
-        int i;
-        char c[sizeof(int)];
-    } u;
-
-    u.i = 1;
-    setProperty(properties, "gnu.cpu.endian",
-                u.c[sizeof(int)-1] == 1 ? "big" : "little");
-#endif
-}
-
 char *getCwd() {
     char *cwd = NULL;
     int size = 256;
@@ -179,22 +150,26 @@ void addDefaultProperties(Object *properties) {
     setProperty(properties, "java.vm.vendor", "Robert Lougher");
     setProperty(properties, "java.vm.vendor.url", "http://jamvm.org");
     setProperty(properties, "java.vm.specification.version", "1.0");
-    setProperty(properties, "java.vm.specification.vendor", "Sun Microsystems, Inc.");
-    setProperty(properties, "java.vm.specification.name", "Java Virtual Machine Specification");
-
+    setProperty(properties, "java.vm.specification.vendor",
+                            "Sun Microsystems, Inc.");
+    setProperty(properties, "java.vm.specification.name",
+                            "Java Virtual Machine Specification");
     setProperty(properties, "java.runtime.version", VERSION);
     setProperty(properties, "java.version", JAVA_COMPAT_VERSION);
     setProperty(properties, "java.vendor", "GNU Classpath");
     setProperty(properties, "java.vendor.url", "http://www.classpath.org");
     setProperty(properties, "java.home", getJavaHome());
     setProperty(properties, "java.specification.version", "1.5");
-    setProperty(properties, "java.specification.vendor", "Sun Microsystems, Inc.");
-    setProperty(properties, "java.specification.name", "Java Platform API Specification");
+    setProperty(properties, "java.specification.vendor",
+                            "Sun Microsystems, Inc.");
+    setProperty(properties, "java.specification.name",
+                            "Java Platform API Specification");
     setProperty(properties, "java.class.version", "48.0");
     setProperty(properties, "java.class.path", getClassPath());
     setProperty(properties, "sun.boot.class.path", getBootClassPath());
     setProperty(properties, "java.boot.class.path", getBootClassPath());
-    setProperty(properties, "gnu.classpath.boot.library.path", getBootDllPath());
+    setProperty(properties, "gnu.classpath.boot.library.path",
+                            getBootDllPath());
     setProperty(properties, "java.library.path", getDllPath());
     setProperty(properties, "java.io.tmpdir", "/tmp");
     setProperty(properties, "java.compiler", "");
@@ -204,9 +179,10 @@ void addDefaultProperties(Object *properties) {
     setProperty(properties, "line.separator", "\n");
     setProperty(properties, "user.name", getenv("USER"));
     setProperty(properties, "user.home", getenv("HOME"));
+    setProperty(properties, "gnu.cpu.endian",
+                            IS_BIG_ENDIAN ? "big" : "little");
 
     setOSProperties(properties);
-    setEndianProperty(properties);
     setUserDirProperty(properties);
     setLocaleProperties(properties);
 }
