@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
- * Robert Lougher <rob@lougher.org.uk>.
+ * Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -20,11 +20,14 @@
  */
 
 #ifndef CREATING
-#include <alloca.h>
 #include <pthread.h>
 #include <setjmp.h>
 #include <stdlib.h>
 
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
+
 /* Thread states */
 
 #define CREATING      0
@@ -95,6 +98,7 @@ struct thread {
 extern Thread *threadSelf();
 extern Thread *jThread2Thread(Object *jThread);
 extern Thread *vmThread2Thread(Object *vmThread);
+extern long long javaThreadId(Thread *thread);
 
 extern void *getStackTop(Thread *thread);
 extern void *getStackBase(Thread *thread);
@@ -131,6 +135,7 @@ extern void detachJNIThread(Thread *thread);
 extern char *getThreadStateString(Thread *thread);
 
 extern Thread *findThreadById(long long id);
+extern Thread *findRunningThreadByTid(int tid);
 extern void suspendThread(Thread *thread);
 extern void resumeThread(Thread *thread);
 
