@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008
- * Robert Lougher <rob@lougher.org.uk>.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
+ * Robert Lougher <rob@jamvm.org.uk>.
  *
  * This file is part of JamVM.
  *
@@ -112,7 +112,7 @@ char *copyUtf8(char *string) {
 
 char *slash2dots(char *utf8) {
     int len = strlen(utf8);
-    char *conv = (char*)sysMalloc(len+1);
+    char *conv = sysMalloc(len+1);
     int i;
 
     for(i = 0; i <= len; i++)
@@ -149,7 +149,7 @@ int utf8CharLen(unsigned short *unicode, int len) {
 
     for(; len > 0; len--) {
         unsigned short c = *unicode++;
-        count += c > 0x7f ? (c > 0x7ff ? 3 : 2) : 1;
+        count += c == 0 || c > 0x7f ? (c > 0x7ff ? 3 : 2) : 1;
     }
 
     return count;
